//---------------------------------------------------------------------------

#include <vcl.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "visa.h"

#pragma hdrstop

#include "frm_main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#define PI 3.141592653589793

//---------------------------------------------------------------------------
Tform_main *form_main;
// -- GLOBALNI PROMENNE -----------------------------------------------------
ViSession defaultRM, vi;
char buf[32768];        // device buffer
char str[80];           // string for reports

double Rs,Ls,Cs,C0,fs,fp,Q;
int bufLength;          // delka bufferu
FILE *f;

//---------------------------------------------------------------------------
__fastcall Tform_main::Tform_main(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall Tform_main::FormCreate(TObject *Sender)
{
    mem_stat->Lines->Clear();

    /* -- INICIALIZACE ZARIZENI -------------------------------*/
    /* Open session to GPIB device at address 18 */
    viOpenDefaultRM (&defaultRM);
    viOpen (defaultRM, "GPIB0::18::INSTR", VI_NULL,VI_NULL, &vi);

    /* Initialize device */
    viPrintf (vi, "*RST\n");

    /* Send an *IDN? string to the device */
    viPrintf (vi, "*IDN?\n");

    /* Read results */
    viScanf (vi, "%t", &buf);

    /* Print identification */
    sprintf(str,"Instrument: %s\n", buf);
    mem_stat->Lines->Add(str);

}
//---------------------------------------------------------------------------
void __fastcall Tform_main::btn_idnClick(TObject *Sender)
{
    /* Vycisteni mem_stat */
    mem_stat->Clear();

    /* Initialize device */
    viPrintf (vi, "*RST\n");
    mem_stat->Lines->Add("Intrument goes to reset.");

    /* Send an *IDN? string to the device */
    viPrintf (vi, "*IDN?\n");

    /* Read results */
    viScanf (vi, "%t", &buf);

    /* Print identification */
    sprintf(str,"Instrument: %s\n", buf);
    mem_stat->Lines->Add(str);
}
//---------------------------------------------------------------------------
void __fastcall Tform_main::FormClose(TObject *Sender,
      TCloseAction &Action)
{
    /* Close session */
    viClose (vi);
    viClose (defaultRM);
}
//---------------------------------------------------------------------------
void __fastcall Tform_main::btn_measureClick(TObject *Sender)
{
  // inicializace GUI pred merenim
  mem_stat->Clear();
  Series1->Clear();
  Series2->Clear();
  Series3->Clear();
  Series4->Clear();
  Sleep(300);

  /* lokalni promenne */
  int flag_phase = 0;   // flag nalezeni faze
  int i = 0;            // index pro cykly
  double cislo;         // vystup
  double fn;

  char *ukazatel;       // ukazatel pro stringove operace
  char *hodnoty, w;

  double Zabs[401];
  double phase[401];
  double sim_Zabs[401];
  double sim_phase[401];

  double A,B,C,D,omega, RE, IM;  // pomocne promenne pro vypocet impedance na zaklade Mansonova obvodu
  double fm45 = 0.0;
  double fp45 = 0.0;
  double MaxValue = 0.0;
  double MinValue = 1e5;
  double RelErr_Zabs = 0.0;
  double RelErr_phase = 0.0;
  

  /* zadani frekvencniho rozsahu */
  double f1 = edt_f1->Text.ToDouble();          // zacatek
  double f2 = edt_f2->Text.ToDouble();          // konec
  double fstep = edt_fstep->Text.ToDouble();    // frekvencni krok
  int N = int ((f2-f1)/fstep);                  // pocet kroku


  /* inicializace zarizeni */
  viPrintf (vi, "*RST\n");
  Sleep(1000);                  // resetuj

  viPrintf(vi,"FUNC:IMP ZTD\n");// definuj
  Sleep(100);

  mem_stat->Lines->Add("STAGE 1 >  ... measuring ...");

  /* ------ STAGE 1 - hruby odhad -------------------------------- */
  while (flag_phase < 4)
  {
      /* cyklus mereni */
      for (i=0; i<(N+1); i++)
      {
          viPrintf (vi,"FREQ %3.0fHZ\n", f1+fstep*i);
          Sleep(300);

          // nacti data
          bufLength = 0;
          viQueryf(vi,"FETC?\n","%#t", &bufLength, buf);

          if (bufLength == 29)  // pokud pristoj meri spravne -> vraceny retezec ma 29 znaku
          {
              ukazatel =  strchr(buf,',');
              *ukazatel = 0;
              cislo = atof((ukazatel+1));
              if(cislo > -45)   // pokud faze je vetsi nez -45 stupnu
              {
                 flag_phase = 6;
                 fn = f1+fstep*(i-1);
                 sprintf(str,"STAGE 1 > frequency: %4.0fkHz",fn/1e3);
                 mem_stat->Lines->Add(str);
                 break;
              }
          }
          else mem_stat->Lines->Add("Chyba pri cteni dat z HP 4285A.\n");
      }
      // resonance nebyla nalezena
      if (flag_phase < 5)
      {
          N = 10*N;
          fstep = fstep/10;
          flag_phase = flag_phase + 1;
          sprintf(str,"STAGE 1 > frequency step: %4.1fkHz",fstep/1e3);
          mem_stat->Lines->Add(str);
      }
  }

  if (flag_phase < 6)
  {
      mem_stat->Lines->Add("Rezonance se pravdepodobne nenachazi v tomto frekvencnim intervalu");
      viClose (vi);
      viClose (defaultRM);
      return;
  }
  else mem_stat->Lines->Add("STAGE 1 > DONE");

  /* ------ STAGE 2 - rozsah ------------------------------------- */
  fstep = fstep/10;                      // zjemni krok
  N = int ((fn+20*fstep - fn)/fstep);    // vypocitej pocet
  f1 = fn;

  flag_phase = 0;

  mem_stat->Lines->Add("STAGE 2 >  ... measuring ...");
  sprintf(str,"STAGE 2 > frequency step: %4.1fkHz",fstep/1e3);mem_stat->Lines->Add(str);
  while (flag_phase < 4)
  {
      /* cyklus mereni */
      for (i=0; i<(N+1); i++)
      {
          viPrintf (vi,"FREQ %3.0fHZ\n", f1+fstep*i);
          Sleep(300);

          //nacti data
          bufLength = 0;
          viQueryf(vi,"FETC?\n","%#t", &bufLength, buf);

          if (bufLength == 29)
          {
              ukazatel =  strchr(buf,',');
              *ukazatel = 0;
              cislo = atof((ukazatel+1));
              if(cislo >= 45)
              {
                 if (flag_phase==0)
                     fn = f1+fstep*i;
                 flag_phase = flag_phase + 1;

                 if  (flag_phase >= 4)
                     break;

              }
          }
      }
      // neni dostatecne jemny krok
      if (flag_phase < 4)
      {
          N = 10*N;
          fstep = fstep / 10;
          flag_phase = 0;
          sprintf(str,"STAGE 2 > frequency step: %4.1fkHz",fstep/1e3);
          mem_stat->Lines->Add(str);
          if (fstep < 100)
             {
             mem_stat->Lines->Add("STAGE 2 > frequency step can not be smaller than 100 Hz.");
             mem_stat->Lines->Add("STAGE 2 > FAILED.");
             return;
             }
      }

  }
  sprintf(str,"STAGE 2 > frequency step: %4.1fkHz",fstep/1e3);mem_stat->Lines->Add(str);
  sprintf(str,"STAGE 2 > frequency: %4.2fkHz",fn/1e3);
  mem_stat->Lines->Add(str);
  mem_stat->Lines->Add("STAGE 2 > DONE");;

  /* ----- STAGE 3 - finalni nalezeni ---------------------------------------- */
  //urceni vrsku
  mem_stat->Lines->Add("STAGE 3 >  ... measuring ...");
  fstep = fstep/10;
  if (fstep < 100)
  {
      mem_stat->Lines->Add("STAGE 3 > frequency step can not be smaller than 100 Hz.");
      fstep = 100;
  }
  N = int ((fn+(200*fstep))-(fn-(200*fstep)))/fstep;
  sprintf(str,"STAGE 3 > frequency step: %4.1fkHz",fstep/1e3);mem_stat->Lines->Add(str);

  /* cyklus mereni */
  for (i=0; i<(N+1); i++)
  {
      viPrintf (vi,"FREQ %3.0fHZ\n", (fn-(200*fstep))+fstep*i);
      Sleep(300);

      //nactena data
      bufLength = 0;
      viQueryf(vi,"FETC?\n","%#t", &bufLength, buf);

      Zabs[i] = 0.0;
      phase[i]= 0.0;
      if (bufLength == 29)
      {
          ukazatel =  strchr(buf,',');
          Zabs[i] = atof(buf);
          f1 = ((fn-(200*fstep))+fstep*i);
          Series1->AddXY(f1/1e3,Zabs[i]);
          *ukazatel = 0;
          phase[i] = atof((ukazatel+1));
          Series2->AddXY(f1/1e3,phase[i]);

          
          /*--- VYHODNOCENI ------*/
          if (Zabs[i] < MinValue){   // Rs a seriova resonance
              MinValue = Zabs[i];
              Rs = Zabs[i];
              fs = f1;
          }
          if (Zabs[i] > MaxValue){   // paralelni resonance
              MaxValue = Zabs[i];
              fp = f1;
          }

          if  (fm45 == 0.0) {        // faze
              if (phase[i] == -45.0)
                  fm45 = f1;
              if (phase[i] > -45.0)
                  fm45 = (-45 - phase[i-1])*fstep/(phase[i] - phase[i-1]) + f1 - fstep;
          }
          if  (fp45 == 0.0) {        // faze
              if (phase[i] == 45.0)
                  fp45 = f1;
              if (phase[i] > 45.0)
                  fp45 = (45 - phase[i-1])*fstep/(phase[i] - phase[i-1]) + f1 - fstep;
          }
      }
  }
  mem_stat->Lines->Add("STAGE 3 > DONE");

  /* ----- VYPOCET ODHADNUTYCH PRVKU --------------------------------------- */
  // urceni Q
  Q = fs/(fp45-fm45);

  // urceni hodnot Cs, Ls a C0
  Cs = 1/(2*PI*fs*Q*Rs);
  Ls = Cs*(Q*Rs)*(Q*Rs);
  C0 = 1/((Ls*(2*PI*fp)*(2*PI*fp))-(1/Cs));

   // vypsani hodnot
  mem_stat->Lines->Add("");
  mem_stat->Lines->Add("  ESTIMATED PARAMETERS  ");
  mem_stat->Lines->Add("______________________________");
  mem_stat->Lines->Add("");
  sprintf(str,"Rs\t: %4.3f Ohms",Rs);mem_stat->Lines->Add(str);
  sprintf(str,"Cs\t: %4.3e F",Cs);mem_stat->Lines->Add(str);
  sprintf(str,"Ls\t: %4.3e H",Ls);mem_stat->Lines->Add(str);
  sprintf(str,"C0\t: %4.3e F",C0);mem_stat->Lines->Add(str);
  mem_stat->Lines->Add("");
  sprintf(str,"Q\t: %4.3f ",Q);mem_stat->Lines->Add(str);
  sprintf(str,"fs\t: %4.3f kHz ",fs/1e3);mem_stat->Lines->Add(str);
  sprintf(str,"fp\t: %4.3f kHz ",fp/1e3);mem_stat->Lines->Add(str);
  sprintf(str,"fm45\t: %4.3f kHz",fm45/1e3);mem_stat->Lines->Add(str);
  sprintf(str,"fp45\t: %4.3f kHz",fp45/1e3);mem_stat->Lines->Add(str);


  /* ----- VYPOCET ODHADNUTYCH PRVKU --------------------------------------- */
  mem_stat->Lines->Add("");
  mem_stat->Lines->Add("SIMULATION > calculating....");
  for (i=0; i<(N+1); i++) {
      omega =  2*PI*((fn-(200*fstep))+fstep*i);
      A = Ls/C0 - 1/(omega*omega*C0*Cs);
      B = Rs/(omega*C0);
      C = Rs;
      D = omega*Ls - 1/(omega*C0)- 1/(omega*Cs);
      RE= (A*C - B*D)/(C*C + D*D);
      IM=-(B*C - A*D)/(C*C + D*D);
      sim_Zabs[i] = sqrt(RE*RE+IM*IM);
      sim_phase[i]= atan(-IM/RE)*180/PI;
      Series3->AddXY(omega/(2e3*PI),sim_Zabs[i]);
      Series4->AddXY(omega/(2e3*PI),sim_phase[i]);
  }
  mem_stat->Lines->Add("SIMULATION > DONE");
  mem_stat->Lines->Add("");


  /* ------ VYPOCET RELATIVNI CHYBY --------------------------------------- */
  RelErr_Zabs = 0.0;
  RelErr_phase= 0.0;
  for (i=0; i<(N+1); i++) {
      RelErr_Zabs = RelErr_Zabs + ((sim_Zabs[i]/Zabs[i]-1)*(sim_Zabs[i]/Zabs[i]-1));
      RelErr_phase= RelErr_phase + ((sim_phase[i]/phase[i]-1)*(sim_phase[i]/phase[i]-1));
  }
  RelErr_Zabs = sqrt(RelErr_Zabs) /(N-1) * 100;
  RelErr_phase= sqrt(RelErr_phase)/(N-1) * 100;
  sprintf(str,"rel. error - phase\t: %4.3f %%",RelErr_phase);mem_stat->Lines->Add(str);
  sprintf(str,"rel. error -  |Z| \t: %4.3f %%",RelErr_Zabs);mem_stat->Lines->Add(str);


}
//---------------------------------------------------------------------------
void __fastcall Tform_main::save_all_dataClick(TObject *Sender)
{
   int N = Series1->LastValueIndex;
   int k;
   FILE *stream;
   char str[80];

   saveDialog->Filter = AnsiString("Text Files|*.txt|All Files|*.*");
   saveDialog->FileName = "";
   saveDialog->Title = AnsiString("Save *.txt file");
   saveDialog->DefaultExt = "txt";


   if (saveDialog->Execute())
   {
       /* open a file for write */
       AnsiString filename = saveDialog->FileName;

       stream = fopen(filename.c_str(), "w");

       if (stream == NULL)
       {
           mem_stat->Lines->Add("??? Cannot open input file.");
           mem_stat->Lines->Add("");

       }
       else
       {
           // zapis odhadnute hodnoty do souboru
           fprintf(stream,"ESTIMATED PARAMETERS  \n");
           fprintf(stream,"______________________________\n");
            fprintf(stream,"Rs\t: %4.5f Ohms\n",Rs);
            fprintf(stream,"Cs\t: %4.5e F\n",Cs);
            fprintf(stream,"Ls\t: %4.5e H\n",Ls);
            fprintf(stream,"C0\t: %4.5e F\n",C0);
            fprintf(stream,"Q\t: %4.5f \n",Q);
            fprintf(stream,"fs\t: %4.5f kHz \n",fs/1e3);
            fprintf(stream,"fp\t: %4.5f kHz \n\n",fp/1e3);

           /* write data to the file */
           fprintf(stream,"frequency, impedance, phase\n");
           fprintf(stream,"______________________________\n");
           for (k = 0; k<N; k++)
               fprintf(stream, "%e, %e, %e\n", Series1->XValue[k],Series1->YValue[k],Series2->YValue[k]);

           /* close the file */
           fclose(stream);

           mem_stat->Lines->Add("Data have been saved.");
       }


   }

}
//---------------------------------------------------------------------------

void __fastcall Tform_main::save_freqClick(TObject *Sender)
{
   int N = Series1->LastValueIndex;
   int k;
   FILE *stream;


   saveDialog->Filter = AnsiString("Text Files|*.txt|All Files|*.*");
   saveDialog->FileName = "";
   saveDialog->Title = AnsiString("Save *.txt file");
   saveDialog->DefaultExt = "txt";


   if (saveDialog->Execute())
   {
       /* open a file for write */
       AnsiString filename = saveDialog->FileName;

       stream = fopen(filename.c_str(), "w");

       if (stream == NULL)
       {
           mem_stat->Lines->Add("??? Cannot open input file.");
           mem_stat->Lines->Add("");

       }
       else
       {
           /* write data to the file */
           for (k = 0; k < N; k++)
               fprintf(stream, "%e\n", Series1->XValue[k]);

           /* close the file */
           fclose(stream);
           mem_stat->Lines->Add("Data have been saved.");
       }
   }
}
//---------------------------------------------------------------------------

void __fastcall Tform_main::save_impClick(TObject *Sender)
{
   int N = Series1->LastValueIndex;
   int k;
   FILE *stream;


   saveDialog->Filter = AnsiString("Text Files|*.txt|All Files|*.*");
   saveDialog->FileName = "";
   saveDialog->Title = AnsiString("Save *.txt file");
   saveDialog->DefaultExt = "txt";


   if (saveDialog->Execute())
   {
       /* open a file for write */
       AnsiString filename = saveDialog->FileName;

       stream = fopen(filename.c_str(), "w");

       if (stream == NULL)
       {
           mem_stat->Lines->Add("??? Cannot open input file.");
           mem_stat->Lines->Add("");

       }
       else
       {
           /* write data to the file */
           for (k = 0; k < N; k++)
               fprintf(stream, "%e\n", Series1->YValue[k]);

           /* close the file */
           fclose(stream);
           mem_stat->Lines->Add("Data have been saved.");
       }
   }
}
//---------------------------------------------------------------------------

void __fastcall Tform_main::save_phaseClick(TObject *Sender)
{
   int N = Series1->LastValueIndex;
   int k;
   FILE *stream;


   saveDialog->Filter = AnsiString("Text Files|*.txt|All Files|*.*");
   saveDialog->FileName = "";
   saveDialog->Title = AnsiString("Save *.txt file");
   saveDialog->DefaultExt = "txt";


   if (saveDialog->Execute())
   {
       /* open a file for write */
       AnsiString filename = saveDialog->FileName;

       stream = fopen(filename.c_str(), "w");

       if (stream == NULL)
       {
           mem_stat->Lines->Add("??? Cannot open input file.");
           mem_stat->Lines->Add("");

       }
       else
       {
           /* write data to the file */
           for (k = 0; k < N; k++)
               fprintf(stream, "%e\n", Series2->YValue[k]);

           /* close the file */
           fclose(stream);
           mem_stat->Lines->Add("Data have been saved.");
       }
   }
}
//---------------------------------------------------------------------------
